<?php
/*  cms-db  Copyright (C) 2007-2010 Jan Hoffmann
    This program comes with ABSOLUTELY NO WARRANTY; for details see /admin/version.php.
    This is free software, and you are welcome to redistribute it under certain conditions; see /admin/version.php for details.  */

if (!defined('cms-db')) {
	include('friendlyerror.firewall.inc.php');
	die();
}

class Archive {

	protected $comment;
	public $error = '';
	protected $dirs = array();
	protected $files = array();

	/* 
	 * Class constructor
	 * Parameters:
	 *  - $comment: optional comment for the archive
	 */
	function __construct( $comment = '' ) {
		$this->comment = $comment;
	}

	/*
	 * Changes the comment of the archive
	 * Parameters:
	 *  - $comment: the new comment
	 */
	public function setComment( $comment ) {
		if ($this->error != '') {
			return false;
		}
		$this->comment = $comment;
	}

	/*
	 * Adds an empty folder to the archive
	 * Parameters:
	 *  - $dest: path of the folder in the archive
	 * Returns:
	 *  true on success, false on error
	 */
	public function addEmptyFolder( $dest ) {
		if ($this->error != '') {
			return false;
		}
		if (substr($dest, 0, 1) == '/' && substr($dest, -1, 1) == '/' && $dest != '/') {
			$this->dirs[] = $dest;
			return true;
		} else {
			$this->error = 'Can\'t add folder.';
			return false;
		}
	}

	/*
	 * Adds a folder
	 * Parameters:
	 *  - $src:  path to the folder to add
	 *  - $dest: path of the folder in the archive
	 * Returns:
	 *  true on success, false on error
	 */
	public function addFolder( $src, $dest ) {
		if ($this->error != '') {
			return false;
		}
		if (substr($dest, 0, 1) == '/' && substr($dest, -1, 1) == '/' && is_dir($src)) {
			if ($dest != '/') {
				$this->dirs[] = $dest;
			}
			$handle = opendir($src);
			while (false !== ($file = readdir($handle))) {
				if ($file != '.' && $file != '..') {
					if (is_file($src.'/'.$file)) {
						$this->files[] = array($src.'/'.$file, $dest.$file);
					} else {
						$this->addFolder($src.'/'.$file, $dest.$file.'/');
					}
				}
			}
			closedir($handle);
			return true;
		} else {
			$this->error = 'Can\'t add folder.';
			return false;
		}
	}

	/*
	 * Generates the archive
	 * Parameters:
	 *  - $filename:    filename for the archive
	 *  - $compression: whether to compress the archive (default on) !!! NOT IMPLEMENTED !!!
	 * Returns:
	 *  true on success, false on error
	 */
	public function getArchive( $filename, $compression = true ) {
		if ($this->error != '') {
			return false;
		}
		if ($compression && function_exists('gzopen')) {
			$fn_open  = 'gzopen';
			$fn_write = 'gzwrite';
			$fn_gets  = 'gzgets';
			$fn_close = 'gzclose';
		} else {
			$fn_open  = 'fopen';
			$fn_write = 'fwrite';
			$fn_gets  = 'fgets';
			$fn_close = 'fclose';
		}
		$n = "\n";
		$handle = call_user_func_array($fn_open, array($filename, 'wb'));
		call_user_func_array($fn_write, array($handle, '== cms -db archive =='.$n));
		call_user_func_array($fn_write, array($handle, $this->comment.$n));
		call_user_func_array($fn_write, array($handle, '=== directories ==='.$n));
		natcasesort($this->dirs);
		foreach($this->dirs as $dir) {
			call_user_func_array($fn_write, array($handle, $dir.$n));
		}
		call_user_func_array($fn_write, array($handle, '=== files ==='.$n));
		foreach($this->files as $file) {
			$content = base64_encode(file_get_contents($file[0]));
			call_user_func_array($fn_write, array($handle, $file[1].'//'.$content.$n));
		}
		call_user_func_array($fn_close, array($handle));
		return true;
	}

}
class unArchive {

	public $comment = '';
	public $error = '';
	protected $archive;
	protected $dirs;
	protected $files;
	protected $handle;
	protected $fn_open;
	protected $fn_write;
	protected $fn_gets;
	protected $fn_close;

	/*
	 * Class constructor
	 * Parameters:
	 *  - $archive: path to the archive
	 */
	function __construct( $archive ) {
		$this->archive = $archive;
	}

	/*
	 * Reads the archive header
	 * Returns:
	 *  true on success, false on error
	 */
	function readHeader() {
		if (file_exists($this->archive)) {
			$n = "\n";
			if (file_get_contents($this->archive, NULL, NULL, 0, 21) == '== cms -db archive ==') {
				$this->fn_open  = 'fopen';
				$this->fn_write = 'fwrite';
				$this->fn_gets  = 'fgets';
				$this->fn_close = 'fclose';
			} else {
				$this->fn_open  = 'gzopen';
				$this->fn_write = 'gzwrite';
				$this->fn_gets  = 'gzgets';
				$this->fn_close = 'gzclose';
			}
			$this->handle = call_user_func_array($this->fn_open, array($this->archive, 'rb'));
			if (call_user_func_array($this->fn_gets, array($this->handle)) == '== cms -db archive =='.$n) {
				while ($line = call_user_func_array($this->fn_gets, array($this->handle))) {
					if ($line == '=== directories ==='.$n) break;
					$this->comment .= $line;
				}
				return true;
			} else {
				$this->error = 'Unsupported file.';
				return false;
			}
		} else {
			$this->error = 'Can\'t open archive.';
			return false;
		}
	}

	/*
	 * Extracts the archive
	 * Parameters:
	 *  - $dest: folder where the archive is extracted to
	 * Returns:
	 *  true on success, false on error
	 */
	public function extract( $dest ) {
		if (!isset($this->handle)) {
			$this->readHeader();
		}
		if ($this->error != '') {
			return false;
		}
		$n = "\n";
		if (is_dir($dest)) {
			while ($line = call_user_func_array($this->fn_gets, array($this->handle))) {
				if ($line == '=== files ==='.$n) break;
				$line = trim($line);
				if (!is_dir($dest.$line)) {
					mkdir($dest.$line);
				}
			}
			while ($line = call_user_func_array($this->fn_gets, array($this->handle))) {
				$line = trim($line);
				$line = explode('//', $line, 2);
				//$filename = $line[0];
				//$content = $line[1];
				if ($line[0] != '') {
					$line[1] = base64_decode($line[1]);
					file_put_contents($dest.$line[0], $line[1]);
				}
			}
			call_user_func_array($this->fn_close, array($this->handle));
			return true;
		} else {
			$this->error = 'Not a directory.';
			return false;
		}
	}

}

?>
